import json
import os

import whisper
from funsketch.sketch.meta import SketchMeta
from .base import BaseTask
from funutil import getLogger

logger = getLogger(__name__)


class TextTask(BaseTask):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def run(self, sketch: SketchMeta, *args, **kwargs):
        os.makedirs(sketch.result_text, exist_ok=True)
        files = os.listdir(sketch.result_audio)
        files = sorted(files, key=lambda x: x)
        model = whisper.load_model("turbo", device="cpu")
        for file in files:
            textfile = os.path.join(sketch.result_text, file.replace(".wav", ".txt"))
            if os.path.exists(textfile):
                logger.info(f"text file {textfile} already exists")
                continue
            audio_path = os.path.join(sketch.result_audio, file)
            result = model.transcribe(audio_path, language="zh", verbose=True)
            with open(textfile, "w", encoding="utf-8") as f:
                f.write(json.dumps(result))



from funsketch.sketch.meta import SketchMeta
from typing import List


class BaseTask:
    def __init__(self, *args, **kwargs):
        pass

    def run(self, sketch: SketchMeta, *args, **kwargs):
        pass


class TaskRun(BaseTask):
    def __init__(self, task_list: List[BaseTask], *args, **kwargs):
        self.task_list = task_list
        super().__init__(*args, **kwargs)

    def run(self, sketch: SketchMeta, *args, **kwargs):
        for task in self.task_list:
            task.run(sketch, *args, **kwargs)



import os

from funsketch.sketch.meta import SketchMeta
from .base import BaseTask
from funutil import getLogger
from moviepy import VideoFileClip

logger = getLogger(__name__)


class AudioTask(BaseTask):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def run(self, sketch: SketchMeta, *args, **kwargs):
        os.makedirs(sketch.result_audio, exist_ok=True)
        files = os.listdir(sketch.result_video)
        files = sorted(files, key=lambda x: x)
        for file in files:
            filepath = os.path.join(sketch.result_video, file)
            if not filepath.endswith(".mp4"):
                continue
            video_path = filepath

            # 指定输出音频文件路径
            audio_path = os.path.join(sketch.result_audio, file.replace(".mp4", ".wav"))
            if os.path.exists(audio_path):
                logger.info(f"audio file {audio_path} already exists")
                continue
            video_clip = VideoFileClip(video_path)
            audio_clip = video_clip.audio
            audio_clip.write_audiofile(audio_path)
            audio_clip.close()
            video_clip.close()
